<?php
namespace Contabilidade\Logic;

use Contabilidade\Entities\Erroctbtemp;
use Contabilidade\Entities\Lancctbtemp;

class Util
{
    const WINDOW_NAME_ERROS_CONTABILIZACAO = 'win_pbi00166_000';

    /*---------    CÓDIGOS DE ERRO DE ACORDO COM A TELA CTB10200 ----------*/
    private static $array_erros = array(
        '001' => 'Total de crédito difere do total de débito.',
        '002' => 'Data fora do período contábil.',
        '003' => 'Partida %s - Conta contábil não cadastrada.',
        '004' => 'Partida %s - Histórico padrão não cadastrado.',
        '005' => 'Partida %s - Código comercial não cadastrado.',
        '006' => 'Partida %s - Valor da partida zerado.',
        '007' => 'Partida %s - Valor de rateio zerado.',
        '008' => 'Partida %s - Filial de rateio não cadastrada.',
        '009' => 'Partida %s - Centro de custo não cadastrado para filial informada.',
        '010' => 'Partida %s - Soma dos rateios difere do valor da partida.',
        '011' => 'Partida %s - Conta contábil sem código comercial informado.',
        '012' => 'Partida %s - Conta Contábil Gerencial sem rateio informado.',
        '013' => 'Partida %s - Centro de custo deve ser informado.',
        '014' => 'Produto %s sem conta contábil.',
        '015' => 'Lançamento fora do período contábil para o financeiro. (BCO, PAG e REC)',
        '016' => 'Lançamento fora do período contábil de documentos fiscais de entrada. (DFE)',
        '017' => 'Lançamento fora do período contábil para documentos fiscais de saída. (DFS)'
    );

    //Verifica se o lançamento possui erros de Contabilizaçã, caso possua
    //retorna true e abre a popUp, se nao retorna false;
    public static function verificaErrosContabilizacao($cdempresa, $nrlancctb)
    {
        $erros = self::getDadosErros($cdempresa, $nrlancctb);
        if ($erros) {
            $dados_erros = array();
            /* @var $erro Erroctbtemp */
            foreach ($erros as $erro) {
                $err      = explode('|', $erro->getCderrotemp());
                $mensagem = self::$array_erros[$err[0]];
                if (strpos($mensagem, '%s')) {
                    $dserro = self::adicionaParametroMsgErro($mensagem, array($err[1]));
                } else {
                    $dserro = $mensagem;
                }
                $dados_erros[] = array('CDEMPRESA'     => $erro->getCdempresa(),
                                       'NRLANCCTBTEMP' => $erro->getNrlancctbtemp(),
                                       'NRSEQCTBTEMP'  => $erro->getNrseqctbtemp(),
                                       'CDERROTEMP'    => $erro->getCderrotemp(),
                                       'DSERRO'        => $dserro);
                tk_set_env_var('DADOS_WINPBI00166', $dados_erros);
                tk_open_window(self::WINDOW_NAME_ERROS_CONTABILIZACAO, true);
                return true;
            }
        } else {
            return false;
        }
    }

    private function getDadosErros($cdempresa, $nrlancctb)
    {
        $erros = Erroctbtemp::getObjectsByWhere(
            " CDEMPRESA   = ? AND NRLANCCTBTEMP   = ?", array($cdempresa,
                $nrlancctb)
        );
        return $erros;
    }

    private static function adicionaParametroMsgErro($mensagem, $param = array())
    {
        foreach ($param as $replace) {
            $mensagem = substr_replace($mensagem, $replace, strpos($mensagem, '%s'), 2);
        }
        return $mensagem;
    }


}

?>