-- =============================================================================
-- Script para corrigir a estrutura da rvore de receitas.
-- Mscara original de produto: 'a.aa.aa.aaa.aa'
-- Nova mscara de produto    : 'a.aa.aa.aaa.aaa'
-- =============================================================================
-- =============================================================================
CREATE TABLE PRODUTO_BKP AS
SELECT * FROM PRODUTO
/
DECLARE
V_RECEITAS   VARCHAR2(2000);
C_EXEC       INTEGER;
V_RESULT     INTEGER;
BEGIN
   /* Verificando se existe mais de uma receita de nvel completo dentro da pastar anterior */
   V_RECEITAS := NULL;
   FOR C_REC IN(SELECT SUBSTR(CDARVPROD,1,8) CDARVPROD,COUNT(*)
                FROM PRODUTO
                WHERE (IDPRODCOMISS = '3') AND
                      (CDNVPRODUTO  = 5  )
                GROUP BY SUBSTR(CDARVPROD,1,8)
                HAVING COUNT(*)>1) LOOP
      V_RECEITAS := V_RECEITAS||C_REC.CDARVPROD||', ';
   END LOOP;
   IF V_RECEITAS IS NOT NULL THEN
      RAISE_APPLICATION_ERROR(-20502,'Existe(m) receita(s) no penltimo nvel que possue(m) mais de uma receita de nvel completo: '||V_RECEITAS,false);
   END IF;
   /**/

   /*Modificando a mascara para ''a.aa.aa.aaa.aaa''*/
   UPDATE PARAMGERAL
   SET CDPICTPROD = 'a.aa.aa.aaa.aaa';

   /*Desativando as constraints*/
   C_EXEC  := DBMS_SQL.OPEN_CURSOR;
   FOR C_DISABLE IN(SELECT 'ALTER TABLE '||TABLE_NAME||' DISABLE CONSTRAINT '||CONSTRAINT_NAME CONSTRAINTS
                    FROM USER_CONSTRAINTS
                    WHERE R_CONSTRAINT_NAME = 'PK_PRODUTO') LOOP

      DBMS_SQL.PARSE(C_EXEC, C_DISABLE.CONSTRAINTS, DBMS_SQL.V7);
      V_RESULT := DBMS_SQL.EXECUTE(C_EXEC);

   END LOOP;
   DBMS_SQL.CLOSE_CURSOR(C_EXEC);
   /**/

   /*Excluindo as receitas de nvel 4*/
	DELETE FROM PRODUTO
   WHERE (CDNVPRODUTO  = '4') AND
         (IDPRODCOMISS = '3');

   /*Passando as receitas de nvel 3 para 4*/
   UPDATE PRODUTO
   SET CDNVPRODUTO = TO_CHAR(CDNVPRODUTO+1),
       CDARVPROD   = 'R0'||DECODE(SUBSTR(CDARVPROD,1,1),'R','1','H','2','U','3',SUBSTR(CDARVPROD,1,1))||SUBSTR(CDARVPROD,2,2)||'0'||SUBSTR(CDARVPROD,4,2)
   WHERE (CDNVPRODUTO  = 3  ) AND
         (IDPRODCOMISS = '3');

   /*Passando as receitas de nvel 1 e 2 para 2 e 3*/
   UPDATE PRODUTO
   SET CDNVPRODUTO = TO_CHAR(CDNVPRODUTO+1),
       CDARVPROD   = 'R0'||DECODE(SUBSTR(CDARVPROD,1,1),'R','1','H','2','U','3',SUBSTR(CDARVPROD,1,1))||SUBSTR(CDARVPROD,2,2)
   WHERE (CDNVPRODUTO  < 3  ) AND
         (IDPRODCOMISS = '3');

   /*Aumentando um caracter nas receitas de nvel completo*/
   UPDATE PRODUTO
   SET CDARVPROD = 'R0'||DECODE(SUBSTR(CDARVPROD,1,1),'R','1','H','2','U','3',SUBSTR(CDARVPROD,1,1))||SUBSTR(CDARVPROD,2,2)||'0'||SUBSTR(CDARVPROD,4,5)
   WHERE (CDNVPRODUTO  = 5  ) AND
         (IDPRODCOMISS = '3');

   /*Aumentando um caracter nos produtos, sub-grupos e servios de nvel completo*/
   UPDATE PRODUTO
   SET CDARVPROD = SUBSTR(CDARVPROD,1,8)||'0'||SUBSTR(CDARVPROD,9,2)
   WHERE (CDNVPRODUTO            = 5  ) AND
         (NVL(IDPRODCOMISS,'0') <> '3');

   /*Criando a receita de primeiro nvel*/
   INSERT INTO PRODUTO
   (CDPRODUTO,CDNVPRODUTO,CDARVPROD,NMPRODUTO,IDCOBTXSERV,IDPRODCOMISS)
   VALUES
   ('X','1','R','Receitas','S','3');

   /*Ativando as constraints*/
   C_EXEC  := DBMS_SQL.OPEN_CURSOR;
   FOR C_ENABLE IN(SELECT 'ALTER TABLE '||TABLE_NAME||' ENABLE CONSTRAINT '||CONSTRAINT_NAME CONSTRAINTS
                    FROM USER_CONSTRAINTS
                    WHERE R_CONSTRAINT_NAME = 'PK_PRODUTO') LOOP

      DBMS_SQL.PARSE(C_EXEC, C_ENABLE.CONSTRAINTS, DBMS_SQL.V7);
      V_RESULT := DBMS_SQL.EXECUTE(C_EXEC);

   END LOOP;
   DBMS_SQL.CLOSE_CURSOR(C_EXEC);
   /**/

   /* Nova alterao */
   UPDATE PRODUTO
   SET CDARVPROD = SUBSTR(CDARVPROD,1,8)||'0'||SUBSTR(CDARVPROD,9,3)
   WHERE (CDNVPRODUTO = 5);

   UPDATE PARAMGERAL
   SET CDPICTPROD = 'a.aa.aa.aaa.aaaa';
END;
/

